import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;
import com.nec.tdd.tools.dbMapper.*;
import com.nec.tdd.tools.logger.*;

public class MyFrame extends JFrame {
   static final int ACT_FBY_PK = 0;
   static final int ACT_FBY_Y = 1;
   static final int ACT_FBY_R = 2;
   static final int ACT_DBY_PK = 3;
   static final int ACT_DBY_A = 4;
   static final int ACT_UBY_X = 5;
   static final int ACT_UBY_Y = 6;
   static final int ACT_DBY_R = 7;
   static final String[] ACTIONS = new String[] {
          "Find a point by primay key (point id)",
          "Find points by y co-ordinate",
          "Find points within a radius",
          /*
          "Move a point to X axis",
          "Move a point to Y axis",
          "Delete points within a radius",
          */
          "Delete a point by primay key (point id)",
          "Delete all"
          };

   JPanel workPanel = null;
   JComponent bottomComponent = null;
   private DBInterface dbInterface = null;
   HashMap pointPanelMap = new HashMap();
   Point lastCreatePoint = null;
   JPopupMenu createPopupMenu = null;
   int pointCounter = 0;
   int lastRadius = -1;

   public MyFrame () {
      super ("DB Mapper Quick Start Demo");
      initialize();
      show();

      this.addWindowListener ( new WindowAdapter() {
         public void windowClosing (WindowEvent e) {
            System.exit(0);
         }
      }
      );
     
     /*
      this.addComponentListener ( new ComponentAdapter() {
         public void componentResized(ComponentEvent e) {
            setSplashScreenBounds ();
         }
      }
      );
     */
   }

   public static void main (String[] args) {
     MyFrame f = new MyFrame ();
   }

   void initialize () {
      int INIT_WIDTH = 500;
      int INIT_HEIGHT = 400;

/*
      MainPanel mp = new MainPanel ();
      mp.setPreferredSize (new Dimension(INIT_WIDTH, INIT_HEIGHT));
      mp.setBackground (Color.white);
*/

      setSize (INIT_WIDTH, INIT_HEIGHT);
      setApplicationIcon();
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      setLocation(screenSize.width/2 - INIT_WIDTH/2, screenSize.height/2 - INIT_HEIGHT/2);

      JLabel necLabel = new JLabel(new ImageIcon(getClass().getResource("/logo.gif")));
      JLabel onsdLabel = new JLabel(new ImageIcon(getClass().getResource("/osg_text.gif")));
      //necLabel.setIconTextGap (24);
      JPanel logoPanel = new JPanel();
      logoPanel.setBackground (Color.white);
      logoPanel.setLayout (new BoxLayout(logoPanel,BoxLayout.X_AXIS));
      logoPanel.add (necLabel);
      logoPanel.add (onsdLabel);

      workPanel = createSplashPanel ();
      // setSplashScreenBounds(ctr, splashPanel);

      JButton startButton = new JButton ("Start Demo");
      startButton.addActionListener (new ActionListener () {
         public void actionPerformed (ActionEvent e) {
            startDemo();
         }
      }
      );
      bottomComponent = startButton;

      Container ctr = this.getContentPane();
      ctr.add (logoPanel, BorderLayout.NORTH);
      ctr.add (workPanel, BorderLayout.CENTER);
      ctr.add (bottomComponent, BorderLayout.SOUTH);
   }

   void showError (String errStr) {
      JOptionPane.showMessageDialog (workPanel,
          errStr, errStr, JOptionPane.ERROR_MESSAGE);
      Logger.error (errStr);
   }

   void showError (String errStr, Exception e) {
      JOptionPane.showMessageDialog (workPanel,
          errStr + "\n" + e.toString(),
          errStr, JOptionPane.ERROR_MESSAGE);
      Logger.error (errStr + " Details: ", e);
   }

   private void startDemo () {
      int option;
      DBModule dbm = null;
      try {
         option = JOptionPane.showConfirmDialog (workPanel,
             "Please make sure the database server is up and running." +
             "\n\n" +
             "Load DBMapper config file <data/dbmapper.xml>" +
             "\nfrom current directory?",
             "Load config file",
             JOptionPane.YES_NO_OPTION,
             JOptionPane.QUESTION_MESSAGE);
         if (option != JOptionPane.YES_OPTION) {
            return;
         }
         dbm = DBModule.init ("./data/dbmapper.xml");
      }
      catch (Exception e) {
         showError("Error loading the DBModule config file \"data/dbmapper.xml\".",
                  e);
         return;
      }

      try {
         option = JOptionPane.showConfirmDialog (workPanel,
             "Validate database connection and mappings?",
             "Validate config file",
             JOptionPane.YES_NO_OPTION,
             JOptionPane.QUESTION_MESSAGE);
         if (option != JOptionPane.YES_OPTION) {
            return;
         }

         dbm.validate (true);
      }
      catch (Exception e) {
         showError("Error initializing the DBModule.", e);
         return;
      }

      try {
         dbInterface = dbm.createDefaultMapper ("default");
      }
      catch (Exception e) {
         showError("No mapper with name <default>.", e);
         return;
      }
      Logger.debug (dbInterface.toString());

      java.sql.ResultSet rs = null;
      try {
         // Initialize point counter
         rs = dbInterface.executeQuery("select max(id) from demo_qs_point");
         if (rs.next()) {
            pointCounter = rs.getInt(1);
            Logger.info ("And the max point id is " + pointCounter);
         }
         try { rs.close(); } catch (Exception eee) {}

         // Find all the points
         Collection points = dbInterface.findAll (Point.class);
         displayPoints (points);
      }
      catch (Exception e) {
         showError("Error creating point screen.", e);
         return;
      }
   }

   private void displayPoints (Collection points) throws Exception {
      JPanel pointPanel = new JPanel();
      createPopupMenu = new JPopupMenu();
      JMenuItem mi = new JMenuItem ("Create new point here");
      mi.addActionListener (new ActionListener() {
         public void  actionPerformed (ActionEvent ae) {
            createNewPoint();
         }
      });
      createPopupMenu.add (mi);
      pointPanel.setLayout (null);
      pointPanel.setBackground (new Color(180, 180, 180));
      pointPanel.addMouseListener (new MouseAdapter() {
         public void  mouseClicked (MouseEvent e) {
            if ((e.getModifiers() & InputEvent.BUTTON3_MASK)
                == InputEvent.BUTTON3_MASK ) {
               lastCreatePoint = new Point (++pointCounter,
                                            e.getX(), e.getY());
               createPopupMenu.show (workPanel, e.getX(), e.getY());
            }
         }
      });

      JPanel controlPanel = new JPanel();
      controlPanel.setLayout (new BoxLayout(controlPanel, BoxLayout.X_AXIS));
      JComboBox actionsCB = new JComboBox (ACTIONS);
      actionsCB.setEditable (false);
      actionsCB.addActionListener (new ActionListener () {
         public void actionPerformed (ActionEvent e) {
            JComboBox c = (JComboBox) e.getSource();
            handleAction (c.getSelectedIndex(), (String) c.getSelectedItem());
         }
      }
      );

      JLabel lbl = new JLabel (" Select Action :   ");
      controlPanel.add (lbl);
      controlPanel.add (actionsCB);

      Container ctr = this.getContentPane();
      ctr.remove (workPanel);
      workPanel = pointPanel;
      ctr.add (workPanel, BorderLayout.CENTER);   // Replace splashPanel
      ctr.remove (bottomComponent);
      bottomComponent = controlPanel;
      ctr.add (bottomComponent, BorderLayout.SOUTH); // Replace demo button
      ctr.validate();
      ctr.repaint();


      // Start drawing points
      if (null != points) {
         Iterator pit = points.iterator();
         while (pit.hasNext()) {
            PointPanel pp = new PointPanel (dbInterface, (Point) pit.next());
            pointPanel.add (pp);
            pointPanelMap.put ("" + pp.getPoint().getId(), pp);
         }
      }
      pointPanel.repaint();
   }

   int promptForIntParam (String field) throws Exception {
      String input = JOptionPane.showInputDialog (
                                workPanel,
                                "Enter " + field,
                                "Enter " + field,
                                JOptionPane.QUESTION_MESSAGE
                              );
      int retVal = -1;
      try {
         retVal = Integer.parseInt (input);
      }
      catch (Exception ex) {
         throw new Exception ("Invalid or null <" + field + "> : " +
                              ex.getClass());
      }
      return retVal;
   }

   void handleAction (int action, String actionDescr) {
      Logger.info ("Handle action : [" + action + "]=" + actionDescr);
      resetPointColors ();
      int inputInt = 0;
      try {
         switch (action) {
            case ACT_FBY_PK:
               inputInt = promptForIntParam ("point id");
               Point p = (Point) dbInterface.findByPrimaryKey (
                                          new Integer(inputInt), Point.class);
               if (null == p) {
                  throw new Exception ("No point with id = " + inputInt);
               }
               Collection cp = new ArrayList();
               cp.add (p);
               colorFoundPoints (cp);
               break;

            case ACT_FBY_Y:
               inputInt = promptForIntParam ("y co-ordinate");
               AttrValMap avMap = new AttrValMap();
               avMap.put ("y", inputInt);
               Collection ca = dbInterface.findByAttributes(avMap, Point.class);
               colorFoundPoints (ca);
               break;

            case ACT_FBY_R:
               inputInt = promptForIntParam ("radius");
               String sql = "select * from demo_qs_point where " +
                            "(col_x*col_x + col_y*col_y) <= " +
                            inputInt*inputInt;
               Collection cs = dbInterface.findByQuery (sql, Point.class);
               colorFoundPoints (cs);
               break;

/*
            case ACT_UBY_X:
               int inputInt = promptForIntParam (input, "point id");
               HashMap avMap = new HashMap(1);
               avMap.put ("y", new Integer(0));
               Collection c = dbInterface.findByAttributes (avMap, Point.class);
               colorFoundPoints (c);
               break;
*/

            case ACT_DBY_PK:
               inputInt = promptForIntParam ("point id");
               if (dbInterface.deleteByPrimaryKey (
                                          new Integer(inputInt), Point.class)) {
                  workPanel.remove ((PointPanel)
                                    pointPanelMap.remove ( "" + inputInt));
               }
               break;

/*
            case ACT_DBY_R:
               int inputInt = promptForIntParam (input, "y co-ordinate");
               HashMap avMap = new HashMap(1);
               avMap.put ("y", new Integer(inputInt));
               Collection c = dbInterface.findByAttributes (avMap, Point.class);
               colorFoundPoints (c);
               break;
*/

            case ACT_DBY_A:
               dbInterface.deleteAll (Point.class);
               Iterator it = pointPanelMap.values().iterator();
               while (it.hasNext()) {
                  PointPanel pp = (PointPanel) it.next();
                  workPanel.remove (pp);
                  it.remove();
               }
               break;
            default:
               break;
         }
      }
      catch (Exception ex) {
         showError ("Error handling action " + actionDescr + ".", ex);
      }
      workPanel.repaint();
   }

   void resetPointColors () {
      Iterator it = pointPanelMap.values().iterator();
      while (it.hasNext()) {
         PointPanel pp = (PointPanel) it.next();
         pp.resetColor();
      }
   }

   void colorFoundPoints (Collection pts) {
      if (null == pts) {
         return;
      }
      Iterator it = pts.iterator();
      while (it.hasNext()) {
         Point p = (Point) it.next();
         PointPanel pp = (PointPanel) pointPanelMap.get ("" + p.getId());
         pp.setFindColor();
      }
   }

   void createNewPoint () {
      try {
         dbInterface.create (lastCreatePoint);
         PointPanel pp = new PointPanel (dbInterface, lastCreatePoint);
         pp.setCreateColor();
         workPanel.add (pp);
         pointPanelMap.put ("" + pp.getPoint().getId(), pp);
         workPanel.repaint();
      }
      catch (Exception ex) {
         showError ("Error creating new point.", ex);
      }
   }

   private JPanel createSplashPanel () {
      Color textColor = new Color (126, 0, 0);
      Color bgColor   = new Color (230, 230, 230);

      JPanel splashPanel = new JPanel();
      splashPanel.setLayout(new BorderLayout());
      splashPanel.setBackground (bgColor);
      // splashPanel.setOpaque(false);

      // caption, version information
      JPanel textPanel = new JPanel();
      textPanel.setOpaque(false);
      //textPanel.setBackground (Color.yellow);
      textPanel.setLayout (new BoxLayout(textPanel,BoxLayout.Y_AXIS));

      JLabel l = new JLabel("DB Mapper Quick Start Demo");
      l.setForeground (textColor);
      l.setFont (new Font("TimesRoman", Font.PLAIN, 24));
      textPanel.add(l);
      JLabel c = new JLabel("NEC ONSD Software Group");
      c.setForeground (textColor );
      c. setFont (new Font("TimesRoman", Font.PLAIN, 10));
      textPanel.add(c);
     
      splashPanel.add (new JLabel("   "), BorderLayout.CENTER);
      splashPanel.add (textPanel, BorderLayout.SOUTH);

      return splashPanel;
   }

  private void setSplashScreenBounds (Component parent, JPanel splashPanel) {
     Dimension panelDim = splashPanel.getPreferredSize();

     splashPanel.setBounds(new Rectangle( (parent.getWidth()-panelDim.width)/2, 
                                          (parent.getHeight()-panelDim.height)/2, 
                                          panelDim.width,
                                          panelDim.height));
  }

  private void setApplicationIcon () {
     MediaTracker tracker = new MediaTracker (this);
     Image im = this.getToolkit().createImage (getClass().getResource("/icon.gif"));
     tracker.addImage (im, 0);

     try {
       tracker.waitForID (0);
     } catch (Exception e) {}

     this.setIconImage (im);
  }
    


   public DBInterface getDBInterface() {
      return dbInterface;
   }



}
